@@ -1,5 +1,5 @@
 #![warn(clippy::future_not_send)]
-#![feature(async_closure, iter_partition_in_place, drain_filter, backtrace, available_concurrency)]
+#![feature(async_closure, iter_partition_in_place, drain_filter, backtrace, available_parallelism)]
 #![doc=include_str!("../readme.md")]
 mod error;
 pub use error::{EyrosError,EyrosErrorKind,Error};
@@ -12,6 +12,7 @@ pub mod tree;
 #[doc(hidden)] pub use tree::{Tree,TreeRef,TreeId,Merge};
 mod bytes;
 mod query;
+pub use query::QTrace;
 mod unfold;
 mod tree_file;
 use tree_file::TreeFile;
@@ -422,8 +423,6 @@ where S: RA, P: Point, V: Value, T: Tree<P,V> {
   }
   /// Query the database for every feature that intersects `bbox`. Results are provided as a
   /// readable stream of `(point,value)` records.
-  /// Queries hold a lock on the database, so you probably shouldn't leave them open for very long
-  /// if you need to make more writes.
   pub async fn query(&mut self, bbox: &P::Bounds) -> Result<query::QStream<P,V>,Error> {
     self.fields.log(&format!["query bbox={:?}", bbox]).await?;
     let mut queries = vec![];
@@ -432,7 +431,34 @@ where S: RA, P: Point, V: Value, T: Tree<P,V> {
         self.fields.log(&format!["query root i={} id={}", i, r.id]).await?;
         let t = self.trees.get(&r.id).await?;
         queries.push(t.lock().await.query(
-          self.trees.clone(), bbox, Arc::clone(&self.fields), i, r.id
+          self.trees.clone(), bbox, Arc::clone(&self.fields), i, r
+        ));
+      }
+    }
+    query::from_queries(queries)
+  }
+  /// Query the database for every feature that intersects `bbox`.
+  /// The provided `trace` will be called right before a tree file is opened with the corresponding
+  /// `TreeRef` for the given tree.
+  pub async fn query_trace(
+    &mut self,
+    bbox: &P::Bounds,
+    trace: Box<dyn query::QTrace<P>>,
+  ) -> Result<query::QStream<P,V>,Error> {
+    self.fields.log(&format!["query bbox={:?}", bbox]).await?;
+    let mut queries = vec![];
+    let trace_r = Arc::new(Mutex::new(trace));
+    for (i,root) in self.meta.read().await.roots.iter().enumerate() {
+      if let Some(r) = root {
+        self.fields.log(&format!["query root i={} id={}", i, r.id]).await?;
+        let t = self.trees.get(&r.id).await?;
+        queries.push(t.lock().await.query_trace(
+          self.trees.clone(),
+          bbox,
+          Arc::clone(&self.fields),
+          i,
+          r,
+          Some(trace_r.clone()),
         ));
       }
     }
